/* **********************************************************************
 * MPS: New version of tps command.
 *
 * select.c
 *
 * Routines for selecting users / boards / processor types / memory sizes
 * from the built in lists.
 *
 * (C) David Beckett 1991, University of Kent at Canterbury
 * **********************************************************************
 */

#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include "types.h"
#include "mzos.h"
#include "inmos.h"
#include "iserver.h"

Search check_mk();
Search check_ht();
Search check_mem();
Search check_mbp();


/* **********************************************************************
 * get_match
 * Extract the information from the given Match and fill in the match
 * fields for use with the select routines.
 * **********************************************************************
 */

BOOL
get_match(fullarg, m)
  char *fullarg;
  Match *m;
{
  char *c;
  char *part;
  char *arg, *origarg;
  Search s;
  int tmp, pos;

  /* Initialise match */
  m->proc_class =MATCH_CHAR_NONE;
  m->proc_type  =MATCH_INT_NONE;
  m->proc_memory=MATCH_INT_NONE;
  m->board_type =MATCH_INT_NONE;
  m->module     =MATCH_INT_NONE;
  m->board      =MATCH_INT_NONE;
  m->processor  =MATCH_INT_NONE;
  m->user_id[0] =MATCH_CHAR_NONE;
  m->process_id =MATCH_INT_NONE;
  m->count      =1;

  /* Get a copy of the argument to work on */
  origarg=(char*)malloc(strlen(fullarg)+1);
  strcpy(origarg, fullarg);
  arg=origarg;

  /* Convert the letters in the Match to lower-case  - easier for later */

  for(c=arg;*c;c++)
    if(isupper(*c))
      *c=tolower(*c);


  /* Initialise flag for main loop */
  s=S_NOT_FOUND;

  /* See if we need <n>x at the start and change the count if so */
#ifdef GRAB
  if(sscanf(arg, "%dx%n", &tmp, &pos)==1)
    {
      if(tmp<0)
	{
	  fprintf(stderr,"%s: Illegal number of multiple claims %d in %s\n",
		  PROGRAM_NAME, tmp, arg);
	  s=S_BAD;
	}
      m->count=tmp;
      arg+=pos;
    }
#endif

  /* Main parsing loop
   * Split up the Match into parts separated by /s and pass the string
   * to the individual routines.
   */

  part=(char*)strtok(arg, ARGUMENT_SEPARATORS);
  while(part && s!=S_BAD)
    {

      /* Check first for 'mk'<num> */
      if(*part=='m' && part[1]=='k')
	s=check_mk(fullarg, part,  m);

      else 
      /* Check for ['h'|'t']<num> */
      if(*part=='h' ||  *part=='t')
	s=check_ht(fullarg, part, m);

      else
      /* Check for [m<x>][b<y>][p<z>] */
      if(*part=='m' || *part=='b' || *part=='p')
	s=check_mbp(fullarg, part, m);

      else
	{
	  /* Check for <num>['m'|'k']] */
	  s=check_mem(fullarg, part, m);
	  if(s==S_BAD)
	    {
#ifdef MPS
	      c=part;
	      if(*c=='.')
		c++; /* :-) If only... */
	      strncpy(m->user_id, c, MAXUSERNAMELEN);
	      s=S_FOUND; /* Well probably... */
#else
	      /* Otherwise is invalid */
	      fprintf(stderr,"%s: Unknown processor specification %s\n",
		      PROGRAM_NAME, part);
	      s=S_BAD;
#endif
	    }
	}

      /* Move to next part */
      part=(char*)strtok(NULL, ARGUMENT_SEPARATORS);
    }

  free(origarg);
  

  /* If invalid - due to parsing error */
  if(s==S_BAD)
    return(TRUE);
  
  return(FALSE);

}


/* **********************************************************************
 * free_matches
 * Free the list of Matchs
 * **********************************************************************
 */

void
free_matches(l)
  List *l;
{
  Match *m,*n;
  
  m=(Match *) l->head;
  while(m!=NULL)
    {
      n=m;
      m=(Match *)(m->node.next);
      free((char*)n);
    }
  return;
}

/* **********************************************************************
 * check_mk
 * Check the mk<num> syntax
 * **********************************************************************
 */

Search
check_mk(fullarg, arg, m)
  char *fullarg;
  char *arg;
  Match *m;
{
  int b_type;
  
  if(sscanf(arg,"mk%d",&b_type)!=1)
    {
      fprintf(stderr,"%s: Illegal board type %s\n",
	      PROGRAM_NAME, arg);
      return(S_BAD);
    }

  if(b_type<0||b_type>MAXBOARDTYPENUMBER)
    {
      fprintf(stderr,"%s: Illegal board number %d in %s\n",
	      PROGRAM_NAME, b_type, arg);
      return(S_BAD);
    }
  
  m->board_type=b_type;
  return(S_FOUND);
}

/* **********************************************************************
 * check_ht
 * Check the [ht]<type> syntax
 * **********************************************************************
 */
  
Search
check_ht(fullarg, arg, m)
  char *fullarg;
  char *arg;
  Match *m;
{
  char p_class;
  char p_ANY;
  int p_type = MATCH_INT_NONE;
  
  /* Scan for '[ht]<type>' */
  if(sscanf(arg,"%c%d",&p_class, &p_type)!=2) {
    if (sscanf(arg,"%c%c",&p_class, &p_ANY) !=2) {
        return(S_BAD);
    }
    else {
       if (p_ANY != 'x'){
          return(S_BAD);
       }  
       p_type = ANY;
    }
  } 
  /* Allow 't4' and 't8' as common abbreviations for 't414' and 't800' */
  if(p_type==4)
    p_type=414;
  else if(p_type==8)
    p_type=800;
  
  /* Check that the type given is valid */
  if(p_type<0||p_type>MAXPROCTYPENUMBER )
    {
      fprintf(stderr,"%s: Illegal processor type %c%d in %s\n",
	      PROGRAM_NAME, p_class, p_type, arg);
      return(S_BAD);
    }

  /* Store class since it is ok */
  m->proc_class=toupper(p_class);
  m->proc_type=p_type;

  return(S_FOUND);
}
	
/* **********************************************************************
 * check_mem
 * Check the <num>[mk] syntax
 * **********************************************************************
 */

Search
check_mem(fullarg, arg, m)
  char *fullarg;
  char *arg;
  Match *m;
{
  int tmp, p_mem;
  char p_mem_char;
  
  /* Scan for '<num>[mk]' */
  tmp=sscanf(arg,"%d%c", &p_mem, &p_mem_char);

  if(tmp<1)
    return(S_BAD);
  
  if(tmp==1)
    {
      fprintf(stderr,
	      "%s: Missing processor memory type in %s (use K or M)\n",
	      PROGRAM_NAME, arg);
      return(S_BAD);
    }

  /* For extended idea, check the memory */
  if(p_mem_char!='k' && p_mem_char!='m')
    {
      fprintf(stderr, "%s: Illegal processor memory type %c in %s\n",
	      PROGRAM_NAME, p_mem_char, arg);
      return(S_BAD);
    }
  

  if(p_mem_char=='m')
    p_mem*=1024;
  
  /* Check that the memory size is sensible */
  if(p_mem<0||p_mem>MAXPROCMEMSIZE)
    {
      fprintf(stderr, "%s: Illegal processor memory size of %dK in %s\n",
	      PROGRAM_NAME, p_mem, arg);
      return(S_BAD);
    }

  /* See if memory has been given without processor type */
  if(m->proc_type==MATCH_INT_NONE)
    {
      fprintf(stderr, "%s: Missing processor type in %s\n",
	      PROGRAM_NAME, fullarg);
      return(S_BAD);
    }

  /* All is valid so store  */
  m->proc_memory=p_mem;

  return(S_FOUND);
  
}

/* **********************************************************************
 * check_mbp
 * Check the [m<num>][b<num>][p<num>] syntax
 * **********************************************************************
 */
  
Search
check_mbp(fullarg, arg, m)
  char *fullarg;
  char *arg;
  Match *m;
{
  int tmp, pos;
  char *origarg, *c;

  c=arg;
  origarg=arg;
  while(*c && (index("mbp",*c)!=NULL))
    switch(*c)
      {
      case 'm':
	tmp=-1;
	if(sscanf(c, "m%d%n", &tmp, &pos)<1||tmp<0||tmp>MAXMODULESPERMACHINE)
	  {
	    fprintf(stderr,
		    "%s: Illegal module number %d in %s\n",
		    PROGRAM_NAME,tmp, origarg);
	    return(S_BAD);
	  }
	if(m->module!=MATCH_INT_NONE)
	  {
	    fprintf(stderr,
		    "%s: Repeated module number %d in %s\n",
		    PROGRAM_NAME,tmp, origarg);
	    return(S_BAD);
	  }
	
	/* Otherwise module number is valid */
	m->module=tmp;
	c+=pos;
	break;
	
      case 'b':
	tmp=-1;
	if(sscanf(c, "b%d%n", &tmp, &pos)<1||tmp<0||tmp>MAXBOARDSPERMODULE)
	  {
	    fprintf(stderr, "%s: Illegal board number %d in %s\n",
		    PROGRAM_NAME,tmp, origarg);
	    return(S_BAD);
	  }
	if(m->board!=MATCH_INT_NONE)
	  {
	    fprintf(stderr, "%s: Repeated board number %d in %s\n",
		    PROGRAM_NAME,tmp, origarg);
	    return(S_BAD);
	  }
	
	/* Assume module 0 if not given */
	if(m->module==MATCH_INT_NONE)
	  m->module=0;
	m->board=tmp;
	c+=pos;
	break;

      case 'p':
	tmp=-1;
	if(sscanf(c, "p%d%n", &tmp, &pos)<1||tmp<0||tmp>MAXPROCSPERBOARD)
	  {
	    fprintf(stderr,
		    "%s: Illegal/repeated processor number %d in %s\n",
		    PROGRAM_NAME,tmp, origarg);
	    return(S_BAD);
	  }
	if(m->processor!=MATCH_INT_NONE)
	  {
	    fprintf(stderr,
		    "%s: Illegal/repeated processor number %d in %s\n",
		    PROGRAM_NAME,tmp, origarg);
	    return(S_BAD);
	  }
	if(m->board==MATCH_INT_NONE) 
	  {
	    fprintf(stderr,
		    "%s: Missing board and/or module number in %s\n",
		    PROGRAM_NAME, origarg);
	    return(S_BAD);
	  }

	m->processor=tmp;
	c+=pos;
	break;

      default:
	fprintf(stderr,"%s: Fallen off switch() in %s with char %c for %s\n",
		PROGRAM_NAME, __FILE__, *c,  origarg);
	return(S_BAD);
      }

  if(*c)
    {
      fprintf(stderr,"%s: Found extra data %s after argument for %s\n",
	      PROGRAM_NAME, c, origarg);
    return(S_BAD);
    }
  else
    return(S_FOUND);
}


/* **********************************************************************
 * print_match
 * Print the fields of the match structure
 * **********************************************************************
 */

void
print_match(stream, m)
  FILE *stream;
  Match *m;
{
  fprintf(stream,
	  "%dx%c%d/%dK",
	  m->count, m->proc_class, m->proc_type, m->proc_memory);
  fprintf(stream," MK%03d Module:%d Board:%d Proc:%d",
	  m->board_type, m->module, m->board, m->processor);
  fprintf(stream," User '%s' ID: %5d\n",
	  m->user_id, m->process_id);
}
